@@ -40,15 +40,15 @@ function test_exception_prepare_stack()
         e = _e;
     }
 
+    Error.prepareStackTrace = undefined;
+
     assert(e.stack.length === 2);
     const f = e.stack[0];
     assert(f.getFunctionName() === 'test_exception_prepare_stack');
-    assert(f.getFileName() === 'tests/test_builtin.js');
+    assert(f.getFileName().endsWith('test_builtin.js'));
     assert(f.getLineNumber() === 38);
     assert(f.getColumnNumber() === 19);
     assert(!f.isNative());
-
-    Error.prepareStackTrace = undefined;
 }
 
 // Keep this at the top; it tests source positions.
@@ -68,16 +68,16 @@ function test_exception_stack_size_limit()
         e = _e;
     }
 
+    Error.stackTraceLimit = 10;
+    Error.prepareStackTrace = undefined;
+
     assert(e.stack.length === 1);
     const f = e.stack[0];
     assert(f.getFunctionName() === 'test_exception_stack_size_limit');
-    assert(f.getFileName() === 'tests/test_builtin.js');
+    assert(f.getFileName().endsWith('test_builtin.js'));
     assert(f.getLineNumber() === 66);
     assert(f.getColumnNumber() === 19);
     assert(!f.isNative());
-
-    Error.stackTraceLimit = 10;
-    Error.prepareStackTrace = undefined;
 }
 
 function assert(actual, expected, message) {
@@ -551,6 +551,9 @@ function test_typed_array()
     a = new Uint16Array(buffer, 2);
     a[0] = -1;
 
+    a = new Float16Array(buffer, 8, 1);
+    a[0] = 1;
+
     a = new Float32Array(buffer, 8, 1);
     a[0] = 1;
 
@@ -768,6 +771,18 @@ function test_regexp()
 
     eval("/[\\-]/");
     eval("/[\\-]/u");
+
+    /* test zero length matches */
+    a = /()*?a/.exec(",");
+    assert(a, null);
+    a = /(?:(?=(abc)))a/.exec("abc");
+    assert(a, ["a", "abc"]);
+    a = /(?:(?=(abc)))?a/.exec("abc");
+    assert(a, ["a", undefined]);
+    a = /(?:(?=(abc))){0,2}a/.exec("abc");
+    assert(a, ["a", undefined]);
+    a = /(?:|[\w])+([0-9])/.exec("123a23");
+    assert(a, ["123a23", "3"]);
 }
 
 function test_symbol()
@@ -985,6 +1000,28 @@ function test_proxy_is_array()
   }
 }
 
+function test_finalization_registry()
+{
+    {
+        let expected = {};
+        let actual;
+        let finrec = new FinalizationRegistry(v => { actual = v });
+        finrec.register({}, expected);
+        queueMicrotask(() => {
+            assert(actual, expected);
+        });
+    }
+    {
+        let expected = 42;
+        let actual;
+        let finrec = new FinalizationRegistry(v => { actual = v });
+        finrec.register({}, expected);
+        queueMicrotask(() => {
+            assert(actual, expected);
+        });
+    }
+}
+
 function test_cur_pc()
 {
     var a = [];
@@ -1052,6 +1089,7 @@ test_weak_set();
 test_generator();
 test_proxy_iter();
 test_proxy_is_array();
+test_finalization_registry();
 test_exception_source_pos();
 test_function_source_pos();
 test_exception_prepare_stack();
